@@ -42,7 +42,7 @@
 #include "gdm.h"
 #include "filecheck.h"
 
-static const gchar RCSid[]="$Id: gdmlogin.c,v 1.28 2001/02/03 22:55:59 jirka Exp $";
+static const gchar RCSid[]="$Id: gdmlogin.c,v 1.29 2001/02/12 07:15:27 jirka Exp $";
 
 static gboolean GdmBrowser;
 static gboolean GdmDebug;
@@ -2040,12 +2040,6 @@ main (int argc, char *argv[])
     bindtextdomain (PACKAGE, GNOMELOCALEDIR);
     textdomain (PACKAGE);
 
-    /* XXX: with our own centering this should not be neccessary, is it? */
-    gnome_preferences_set_dialog_position (GTK_WIN_POS_CENTER);
-    
-    bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-    textdomain (PACKAGE);
-
     gdm_login_parse_config ();
 
     gdm_screen_init ();
